@@ -1,3 +1,6 @@
+__all__ = ['LinkPreview', 'TelegramPost', 'Channel', 'TelegramChannelScraper']
+
+
 import bs4
 import dataclasses
 import datetime
@@ -8,7 +11,7 @@ import typing
 import urllib.parse
 
 
-logger = logging.getLogger(__name__)
+_logger = logging.getLogger(__name__)
 _SINGLE_MEDIA_LINK_PATTERN = re.compile(r'^https://t\.me/[^/]+/\d+\?single$')
 
 
@@ -36,7 +39,7 @@ class TelegramPost(snscrape.base.Item):
 
 
 @dataclasses.dataclass
-class Channel(snscrape.base.Entity):
+class Channel(snscrape.base.Item):
 	username: str
 	title: str
 	verified: bool
@@ -84,7 +87,7 @@ class TelegramChannelScraper(snscrape.base.Scraper):
 			dateDiv = post.find('div', class_ = 'tgme_widget_message_footer').find('a', class_ = 'tgme_widget_message_date')
 			rawUrl = dateDiv['href']
 			if not rawUrl.startswith('https://t.me/') or sum(x == '/' for x in rawUrl) != 4 or rawUrl.rsplit('/', 1)[1].strip('0123456789') != '':
-				self.logger.warning(f'Possibly incorrect URL: {rawUrl!r}')
+				_logger.warning(f'Possibly incorrect URL: {rawUrl!r}')
 			url = rawUrl.replace('//t.me/', '//t.me/s/')
 			date = datetime.datetime.strptime(dateDiv.find('time', datetime = True)['datetime'].replace('-', '', 2).replace(':', ''), '%Y%m%dT%H%M%S%z')
 			if (message := post.find('div', class_ = 'tgme_widget_message_text')):
@@ -120,14 +123,14 @@ class TelegramChannelScraper(snscrape.base.Scraper):
 					if imageI['style'].startswith("background-image:url('"):
 						kwargs['image'] = imageI['style'][22 : imageI['style'].index("'", 22)]
 					else:
-						self.logger.warning(f'Could not process link preview image on {url}')
+						_logger.warning(f'Could not process link preview image on {url}')
 				linkPreview = LinkPreview(**kwargs)
 			yield TelegramPost(url = url, date = date, content = content, outlinks = outlinks, linkPreview = linkPreview)
 
 	def get_items(self):
 		r, soup = self._initial_page()
 		if '/s/' not in r.url:
-			logger.warning('No public post list for this user')
+			_logger.warning('No public post list for this user')
 			return
 		while True:
 			yield from self._soup_to_items(soup, r.url)
@@ -148,7 +151,7 @@ class TelegramChannelScraper(snscrape.base.Scraper):
 			raise snscrape.base.ScraperException(f'Got status code {r.status_code}')
 		soup = bs4.BeautifulSoup(r.text, 'lxml')
 		membersDiv = soup.find('div', class_ = 'tgme_page_extra')
-		if membersDiv.text.endswith(' members'):
+		if membersDiv.text.endswith(' subscribers'):
 			kwargs['members'] = int(membersDiv.text[:-8].replace(' ', ''))
 		kwargs['photo'] = soup.find('img', class_ = 'tgme_page_photo_image').attrs['src']
 
@@ -166,7 +169,7 @@ class TelegramChannelScraper(snscrape.base.Scraper):
 			kwargs['username'] = next(self._soup_to_items(soup, r.url, onlyUsername = True))
 		except StopIteration:
 			# If there are no posts, fall back to the channel info div, although that should never happen due to the 'Channel created' entry.
-			logger.warning('Could not find a post; extracting username from channel info div, which may not be capitalised correctly')
+			_logger.warning('Could not find a post; extracting username from channel info div, which may not be capitalised correctly')
 			kwargs['username'] = channelInfoDiv.find('div', class_ = 'tgme_channel_info_header_username').text[1:] # Remove @
 		if (descriptionDiv := channelInfoDiv.find('div', class_ = 'tgme_channel_info_description')):
 			kwargs['description'] = descriptionDiv.text
@@ -192,9 +195,9 @@ class TelegramChannelScraper(snscrape.base.Scraper):
 		return Channel(**kwargs)
 
 	@classmethod
-	def setup_parser(cls, subparser):
+	def _cli_setup_parser(cls, subparser):
 		subparser.add_argument('channel', type = snscrape.base.nonempty_string('channel'), help = 'A channel name')
 
 	@classmethod
-	def from_args(cls, args):
-		return cls._construct(args, args.channel)
+	def _cli_from_args(cls, args):
+		return cls._cli_construct(args, args.channel)
